<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class Order {

    private $orderService;
    private $util;

    public function __construct(
        \App\Service\Order $orderService,
        \App\Util\Util $util
    ) {

        $this->orderService = $orderService;
        $this->util = $util;
    }

    public function history(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            
            if (empty($params['dtInicial']) || empty($params['dtFinal'])) {
                throw new \Exception("Data inicial/final não informada.");
            }

            $dataorder = $this->orderService->history($params);

            if(empty($dataorder)) {
                throw new \Exception("Nenhum pedido encontrado.");
            }

            foreach($dataorder as $row) {
                $row['IDORGCMDVENDA'] = $this->util->commandSourceDictionary($row['IDORGCMDVENDA']);
                $dataset[] = $row;
            }

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $dataset,
                'pagination' => [
                    'limit' => $params['limit'] ?? 50,
                    'total' => count($dataset),
                    'page' => $params['page'] ?? 1
                ]
            ]));
    
            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

   
    public function cancel(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();
            $cdFilial    = !empty($params['CDFILIAL']) ? $params['CDFILIAL'] : null;
            $cdOperador  = !empty($params['CDOPERADOR']) ? $params['CDOPERADOR'] : null;
            $nrComanda   = !empty($params['NRCOMANDA']) ? $params['NRCOMANDA'] : null;
            $nrVendaRest = !empty($params['NRVENDAREST']) ? $params['NRVENDAREST'] : null;
            $dsCancMotiv = !empty($params['DSCANCMOTIV']) ? $params['DSCANCMOTIV'] : null;

            if(empty($cdFilial) || empty($cdOperador) || empty($nrComanda) || empty($nrVendaRest) || empty($dsCancMotiv)) {
                throw new \Exception("Algum parâmetro não foi informado.");
            }

            $resultCancel = $this->orderService->cancel($cdFilial, $cdOperador, $nrComanda, $nrVendaRest, $dsCancMotiv);

            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => $resultCancel,
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getBranchList(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            $search = isset($params['search']) ? strval(strtoupper($params['search'])) : '';

            $branchList = $this->orderService->getBranchList();

            if(empty($branchList)) {
                throw new \Exception("Nenhuma filial encontrada.");
            }

            if (!empty($search) && $search !== '') {
                $branchList = array_values(array_filter($branchList, function($value) use ($search) {
                    return (mb_strpos($value['LABELFILIAL'], $search) !== false) || '';
                }));
            }

            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => '',
                'data'    => $branchList,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($branchList),
                    'page' => $params['page'] ?? 1
                ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getOrderNumber(Request $request, Response $response) {
        try {
            $params = $request->getQueryParams();
            $search = isset($params['search']) ? strval($params['search']) : '';
            $orderNumber = [];

            if (!empty($search) && strlen($search) > 4) {
                $orderNumber = $this->orderService->getOrderNumber($search);
            }

            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => '',
                'data'    => $orderNumber,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($orderNumber),
                    'page' => $params['page'] ?? 1
                    ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getConsumers(Request $request, Response $response) 
    {
        try {
            $params = $request->getQueryParams();

            $consumers = $this->orderService->getConsumers($params);

            if (isset($consumers)) {
            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => '',
                'data'    => $consumers,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($consumers),
                    'page' => $params['page'] ?? 1
                ],
            ]));
        }
            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getPaymentMethods(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $paymentMethods = $this->orderService->getPaymentMethods($params);
            
            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $paymentMethods
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getOrderItemsRepo(Request $request, Response $response) {
        try {
			$params   = $request->getParsedBody();
            $products = $this->orderService->getOrderItemsRepo($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $products
            ]));

            return $response->withHeader('Content-Type', 'application/json');
		} catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getItemObservation(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            $itemObservation = $this->orderService->getItemObservation($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $itemObservation
            ]));

            return $response->withHeader('Content-Type', 'application/json');
		} catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function forwardOrder(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $cdFilial   = !empty($params['CDFILIAL']) ? $params['CDFILIAL'] : null;
            $nrComanda = !empty($params['NRCOMANDA']) ? $params['NRCOMANDA'] : null;
            $nrVendaRest = !empty($params['NRVENDAREST']) ? $params['NRVENDAREST'] : null;

            if(empty($cdFilial) || empty($nrComanda) || empty($nrVendaRest)) {
                throw new \Exception("Algum parâmetro não foi informado.");
            }
  
            $forwardOrder = $this->orderService->forwardOrder($cdFilial, $nrComanda, $nrVendaRest);
  
            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => 'Pedido adiantado com sucesso.',
                'data' => $forwardOrder
            ]));

            return $response->withHeader('Content-Type', 'application/json');
          } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
      }
  

}