<?php

namespace App\Factory;

use Doctrine\DBAL\Connection;

class Layout {
  private $connection;

  public function __construct(Connection $connection) {
      $this->connection = $connection;
  }
  
  public function fixOldLayout() {
    $sql = "UPDATE RTLCONTROLLYT SET NMTAB = 'PEDIDO' WHERE NMTAB IS NULL OR NMTAB = 'P'";
    $this->connection->executeUpdate($sql);
  }

  public function getLayout($params) {
    $this->fixOldLayout();
    $sql = "
            SELECT * FROM RTLCONTROLLYT WHERE NMTAB = :NMTAB
        ";

		$stmt = $this->connection->prepare($sql);
    $stmt->bindValue('NMTAB', $params['NMTAB']);
		$result = $stmt->executeQuery();
		return $result->fetchAllAssociative();
  }

  public function updateLayout($params) {
    $sql = "
            UPDATE RTLCONTROLLYT
              SET IDATIVO = CASE WHEN NMLAYOUT = :NMLAYOUT THEN :IDATIVO ELSE 'N' END,
              DSCOLUNAS = COALESCE(:DSCOLUNAS, DSCOLUNAS)
            WHERE NMLAYOUT = :NMLAYOUT
              AND NMTAB    = :NMTAB
        ";

    $stmt = $this->connection->executeUpdate($sql, [
      "NMLAYOUT"  => $params['NMLAYOUT'],
      "DSCOLUNAS" => $params['DSCOLUNAS'],
      "IDATIVO"   => $params['IDATIVO'],
      "NMTAB"     => $params['NMTAB'],
    ]);
    return $stmt;
  }

  public function saveLayout($params) {
    $sql = "
            INSERT INTO RTLCONTROLLYT
              (NMLAYOUT, DSCOLUNAS, IDATIVO, CDOPERADOR, NMTAB)
              VALUES (:NMLAYOUT, :DSCOLUNAS, :IDATIVO, :CDOPERADOR, :NMTAB)
        ";

    $stmt = $this->connection->executeUpdate($sql, [
      "NMLAYOUT"   => $params['NMLAYOUT'],
      "DSCOLUNAS"  => $params['DSCOLUNAS'],
      "IDATIVO"    => $params['IDATIVO'],
      "CDOPERADOR" => $params['CDOPERADOR'],
      "NMTAB"      => $params['NMTAB'],
    ]);

    return $stmt;
  }

  public function deleteLayout($params) {
    $sql = "
            DELETE RTLCONTROLLYT
              WHERE NMLAYOUT = :NMLAYOUT
                AND NMTAB    = :NMTAB
        ";

    $stmt = $this->connection->executeUpdate($sql, [
      "NMLAYOUT" => $params['NMLAYOUT'],
      "NMTAB"    => $params['NMTAB'],
    ]);

    return $stmt;
  }
}