<?php

namespace App\Factory;

use Doctrine\DBAL\Connection;

class LogTracking
{
    private $connection;

    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }

    public function errorLogs($paramsLog) {

		$sql = "
		SELECT 
    COALESCE(F.CDFILIAL, 'N/A') AS CDFILIAL, 
    COALESCE(F.NMFILIAL, 'N/A') AS NMFILIAL, 
    I.NMINTEGRACAO,
    TO_CHAR(E.DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS') AS DTINCLUSAO, 
    COALESCE(E.DSINTEGRACAOERR, '-') as DSINTEGRACAOERR, 
    E.CDINTEGRACAOERR
	FROM 
    INTEGRACAOERR E
    LEFT JOIN FILIAL F ON E.CDFILIAL = F.CDFILIAL
    JOIN INTEGRACAO I ON E.IDINTEGRACAO = I.CDINTEGRACAO
	WHERE 
    E.IDATIVO = 'S'
    AND ((INSTR(:CDFILIAL, E.CDFILIAL) > 0) OR :CDFILIAL IS NULL)
    AND ((INSTR(:CDINTEGRACAO, I.CDINTEGRACAO) > 0) OR :CDINTEGRACAO IS NULL)
    AND (:STARTDATE IS NULL OR (TRUNC(E.DTINCLUSAO) BETWEEN TO_DATE(:STARTDATE, 'DD/MM/YYYY') AND TO_DATE(:ENDDATE, 'DD/MM/YYYY')))
    AND (
        :SEARCH IS NULL
        OR (
            LOWER(E.CDFILIAL) LIKE LOWER(:SEARCH)
            OR LOWER(F.NMFILIAL) LIKE LOWER(:SEARCH)
            OR LOWER(I.NMINTEGRACAO) LIKE LOWER(:SEARCH)
            OR LOWER(TO_CHAR(E.DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS')) LIKE LOWER(:SEARCH)
            OR LOWER(E.DSINTEGRACAOERR) LIKE LOWER(:SEARCH)
        )
    )
		 {$paramsLog['order']}
		";
		$stmt = $this->connection->prepare($sql);

		if (!is_null($paramsLog['search'])) {
			$search = $paramsLog['search'];
			$stmt->bindValue("SEARCH", "%$search%");
		}

		$stmt->bindValue("CDFILIAL", $paramsLog['cdFilial']);
		$stmt->bindValue("CDINTEGRACAO",$paramsLog['cdIntegracao']);
		$stmt->bindValue("STARTDATE", $paramsLog['dtInclusao' ][0]);
		$stmt->bindValue("ENDDATE", $paramsLog['dtInclusao' ][1]);
		$result = $stmt->executeQuery();
		return $result->fetchAllAssociative();
	}

    public function deleteOldLogs($deleteLogMonths) 
	{
		$sql = "	
		DELETE
		FROM 
			INTEGRACAOERR
		WHERE DTINCLUSAO NOT BETWEEN (TO_DATE(:DELETEMONTH, 'DD/MM/YYYY HH24:MI:SS')) AND (TO_DATE(:CURRENTDAY, 'DD/MM/YYYY HH24:MI:SS'))
		";
		$stmt = $this->connection->prepare($sql);
		$deleteMonth = mktime(0, 0, 0, date("m")-$deleteLogMonths, date("d"),   date("Y"));
		$deleteMonth = date("d-m-Y H:i:s", $deleteMonth);
		$currentDay = date("d-m-Y H:i:s");
		$stmt->bindValue(":DELETEMONTH", $deleteMonth);
		$stmt->bindValue(":CURRENTDAY", $currentDay);
		$stmt->executeQuery();
	}

    public function updateOldLogs($updateLogMonths)
	{
		$sql = "
			UPDATE INTEGRACAOERR
			SET IDATIVO = 'N'
			WHERE DTINCLUSAO NOT BETWEEN (TO_DATE(:UPDATEMONTH, 'DD/MM/YYYY HH24:MI:SS')) AND (TO_DATE(:CURRENTDAY, 'DD/MM/YYYY HH24:MI:SS'))
		";
		$stmt = $this->connection->prepare($sql);
		$updateMonth = mktime(0, 0, 0, date("m")-$updateLogMonths, date("d"),   date("Y"));
		$updateMonth = date("d-m-Y H:i:s", $updateMonth);
		$currentDay = date("d-m-Y H:i:s");
		$stmt->bindValue(":UPDATEMONTH", $updateMonth);
		$stmt->bindValue(":CURRENTDAY", $currentDay);
		$stmt->executeQuery();
	}

    public function deleteLogs($CDINTEGRACAOERR)
    {
        $sql = "
		UPDATE INTEGRACAOERR
		SET IDATIVO = 'N'
		WHERE CDINTEGRACAOERR = :CDINTEGRACAOERR
		";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("CDINTEGRACAOERR", $CDINTEGRACAOERR);
        $stmt->executeQuery();
    }

    public function selectTableInteg() {
		$sql = "
		SELECT NMINTEGRACAO, CDINTEGRACAO
		FROM INTEGRACAO
		";

		$stmt = $this->connection->prepare($sql);

		$result = $stmt->executeQuery();
		
		return $result->fetchAllAssociative();
	}

	public function selectTableFilial() {
		$sql = "
		SELECT CDFILIAL, NMFILIAL 
		FROM FILIAL
		ORDER BY CDFILIAL
		";

		$stmt = $this->connection->prepare($sql);

		$result = $stmt->executeQuery();
		
		return $result->fetchAllAssociative();
	}

	public function getLogDetails($cdIntegracaoErr) {
		$sql = "
			SELECT 
				DSARQPED
			FROM
				INTEGRACAOERR
			WHERE
				CDINTEGRACAOERR = :CDINTEGRACAOERR   
		";
	
		$stmt = $this->connection->prepare($sql);
		$stmt->bindValue("CDINTEGRACAOERR", $cdIntegracaoErr);
		$result = $stmt->executeQuery();
		
		return $result->fetchAllAssociative();
	}
}
