<?php

namespace App\Service;

class Consumer {

    private $util;
    private $consumerFactory;
    private $branchFactory;
    private $addressFactory;

    public function __construct(\App\Util\Util $util, \App\Factory\Consumer $consumerFactory, \App\Factory\Branch $branchFactory, \App\Factory\Address $addressFactory) {
        $this->util = $util;
        $this->consumerFactory = $consumerFactory;
        $this->branchFactory = $branchFactory;
        $this->addressFactory = $addressFactory;
    }

    public function getConsumersByPhone($NRTELEFONE) {
        $dadosCliente = $this->consumerFactory->getConsumersByPhone($NRTELEFONE);
        
        if(empty($dadosCliente)) {
            return false;
        }
		
        $muid = 0;
		foreach ($dadosCliente as &$cliente){
			$cliente['NRCEPCONS']	= str_replace('-', '', $cliente['NRCEPCONS']);
     		$cliente['DTNASCCONS']  = !empty($cliente['DTNASCCONS']) ? (substr($cliente['DTNASCCONS'],8,2).'/'.substr($cliente['DTNASCCONS'],5,2).'/'.substr($cliente['DTNASCCONS'],0,4)) : null;
    		$cliente['BUILTSTRING'] = $cliente['NMCONSUMIDOR']." / ".$cliente['DSENDECONS'].", ".$cliente['NRENDECONS'];
    		$cliente['MULTIUSERID'] = $muid++;
    	}

        return $dadosCliente;
    }

    public function getConsumerAddresses($CDCLIENTE, $CDCONSUMIDOR) {
        return $this->consumerFactory->getConsumerAddresses($CDCLIENTE, $CDCONSUMIDOR);
    }

	public function getStandardClient($cdFilial) {
		$standardClient = $this->branchFactory->getStandardClient($cdFilial);
		if(empty($standardClient['CDCLIENTE'])) {
            throw new \Exception("Cliente padrão não encontrado.");
        }
		return $standardClient;
	}

    public function addConsumer($consumerData) {
		$standardClient = $this->getStandardClient($consumerData['CDFILIAL']);
		$consumerData['CDCLIENTE'] = $standardClient['CDCLIENTE'];
		$nmBairro = $this->util->removeAcentos($consumerData['NMBAIRRO']);
		$cdBairro = $this->addressFactory->getNeighbourhoodCode($nmBairro);

		if (empty($cdBairro['CDBAIRRO'])) {
			throw new \Exception("Bairro não encontrado.");
		}

		$consumerData['CDBAIRRO'] = $cdBairro['CDBAIRRO'];
		$nmMunicipio = $this->util->removeAcentos($consumerData['NMMUNICIPIO']);
		$cdMunicipio = $this->addressFactory->getCityCode($nmMunicipio);

		if (empty($cdMunicipio['CDMUNICIPIO'])) {
			throw new \Exception("Municipio não encontrado.");
		}

		$consumerData['CDMUNICIPIO'] = $cdMunicipio['CDMUNICIPIO'];
		$consumerData['NRCPFRESPCON'] = $this->util->formatsCPF($consumerData['NRCPFRESPCON']);
		$consumerData['CDCONSUMIDOR'] = $this->util->newCode('CONSUMIDOR'.$consumerData['CDCLIENTE'], 18);
		$CDSENHACONSMD5 = isset($consumerData['CDSENHACONSMD5']) ? $this->util->encrypt($consumerData['CDSENHACONSMD5']) : null;

		if(!empty($consumerData['VENDORDETAILS'])) {
			$consumerData['CDCONSUMIDOR'] = 'CDE'.$consumerData['CDCONSUMIDOR'];
			$dsinfocons = 'CDE - Incluido';
		} else {
			$consumerData['CDCONSUMIDOR'] = 'SDE'.$consumerData['CDCONSUMIDOR'];
			$dsinfocons = 'SDE - Incluido';
		}

		$tiporecebimento = $this->branchFactory->getTipoRec($consumerData['CDFILIAL']);
		$idatuconsumi	 = 'S';
		$nmrespcons 	 = $consumerData['NMCONSUMIDOR'];
		$nmlogincons	 = substr($consumerData['NMCONSUMIDOR'], 0, 30);
		$idsitconsumi	 = '1';
		$cdidconsumid	 = $consumerData['CDCONSUMIDOR'];
		$nrramal1cons	 = '';
		$nrtele2cons	 = '';
		$nrramal2cons	 = '';
		$nrfaxcons		 = '';
		$idconsumidor	 = 'C';
		$cdvendedor 	 = '';
		$idsexocons 	 = 'M';
		$idtpvendacons	 = '7';
		$idimpcpfcupom	 = 'S';
		$idcadconflibcon = 'N';
		$idperconsprodex = 'S';
		$idtpselmanha	 = 'N';
		$idtpsealmoco	 = 'N';
		$idtpseltarde	 = 'N';
		$idcrachamestre  = 'N';

		$date = explode('/', $consumerData['DTNASCCONS']);

		$streetAddress = $consumerData['DSENDECONS'];
		$neighborhood  = $consumerData['NMBAIRRO'];
		$cityName      = $consumerData['NMMUNICIPIO'];
		$stateCode     = strlen($consumerData['SGESTADO']) > 2 ? $this->util->mapState($consumerData['SGESTADO']) : $consumerData['SGESTADO'];
		$userLat = null;
		$userLng = null;

		try {
			if($streetAddress && $neighborhood && $cityName && $stateCode) {
				$resp = $this->util->getCoordinatesByAddress($streetAddress, $consumerData['NRENDECONS'], $neighborhood, $cityName, $stateCode);
				if($resp['status'] == 'OK') {
					$location  = $resp['results'][0]['geometry']['location'];
					$userLat   = $location['lat'];
					$userLng   = $location['lng'];
				}
			}
		} catch(\Exception $e) {
			throw new \Exception ($e->getMessage());
		}
		$params = array(
			'CDCLIENTE'       => $consumerData['CDCLIENTE'],
			'CDCONSUMIDOR'    => $consumerData['CDCONSUMIDOR'],
			'NMCONSUMIDOR'    => $consumerData['NMCONSUMIDOR'],
			'IDSITCONSUMI'    => $idsitconsumi,
			'CDCCUSCLIE'      => $tiporecebimento[0]['CDCCUSCLIE'],
			'IDCONSUMIDOR'    => $idconsumidor,
			'IDATUCONSUMI'    => $idatuconsumi,
			'IDTPVENDACONS'   => $idtpvendacons,
			'IDIMPCPFCUPOM'   => $idimpcpfcupom,
			'IDCADCONFLIBCON' => $idcadconflibcon,
			'IDPERCONSPRODEX' => $idperconsprodex,
			'IDTPSELMANHA'	  => $idtpselmanha,
			'IDTPSEALMOCO'	  => $idtpsealmoco,
			'IDTPSELTARDE'	  => $idtpseltarde,
			'IDCRACHAMESTRE'  => $idcrachamestre,
			'SGESTADO'		  => $stateCode,
			'CDMUNICIPIO'	  => $consumerData['CDMUNICIPIO'],
			'CDTIPOCONS'	  => $tiporecebimento[0]['CDTIPOCONS'],
			'CDIDCONSUMID'	  => $cdidconsumid,
			'DSEXIBENDE'	  => $consumerData['DSEXIBENDE'],
			'DSENDECONS'	  => substr($consumerData['DSENDECONS'], 0, 50),
			'NMBAIRCONS'	  => $this->formatNeightbor($consumerData['NMBAIRRO']),
			'NRCEPCONS' 	  => $consumerData['NRCEPCONS'],
			'NRTELECONS'	  => $consumerData['NRTELECONS'],
			'NRRAMAL1CONS'	  => $nrramal1cons,
			'NRTELE2CONS'	  => $nrtele2cons,
			'NRRAMAL2CONS'	  => $nrramal2cons,
			'NRFAXCONS' 	  => $nrfaxcons,
			'DSEMAILCONS'	  => $consumerData['DSEMAILCONS'],
			'CDVENDEDOR'	  => $cdvendedor,
			'NRRGCONSUMID'	  => $consumerData['NRRGCONSUMID'],
			'NRCPFRESPCON'	  => $consumerData['NRCPFRESPCON'],
			'NMRESPCONS'	  => $nmrespcons,
			'NMLOGINCONS'	  => $nmlogincons,
			'DSINFOCONS'	  => $dsinfocons,
			'IDSEXOCONS'	  => $idsexocons,
			'DTNASCCONS'	  => $consumerData['DTNASCCONS'] == '' ? null : $date[2].'-'.$date[1].'-'.$date[0],
			'DSCOMPLENDECONS' => $consumerData['DSCOMPLENDECONS'],
			'DSREFERENDECONS' => $consumerData['DSREFERENDECONS'],
			'NRCELULARCONS'   => $consumerData['NRCELULARCONS'],
			'CDBAIRRO'	      => $consumerData['CDBAIRRO'],
			'NRENDECONS'	  => $consumerData['NRENDECONS'],
			'CDSENHACONSMD5'  => $CDSENHACONSMD5,
			'CDCONSFACEBOOK'  => $consumerData['CDCONSFACEBOOK'] ?? null,
			'DSENDEIMGCONS'   => $consumerData['DSENDEIMGCONS'] ?? null,
			'CDTOKENCONSFB'   => $consumerData['CDTOKENCONSFB'] ?? null,
			'DTEXPTOKENFB'	  => $consumerData['DTEXPTOKENFB'] ?? null,
			'NRLATCONS' 	  => $userLat,
			'NRLONGCONS'	  => $userLng
		);
        
		$this->consumerFactory->addConsumer($params);

		$consumer = $this->consumerFactory->getConsumerById($consumerData['CDCLIENTE'], $consumerData['CDCONSUMIDOR']);
		return $consumer;
	}

	public function formatNeightbor($neightbor) {
		$formatNeighbor = $neightbor;
		$dictionary = self::dictionary;
		
		if (strlen($neightbor) >= 30){
			$formatNeighbor = $this->removeAcentos($formatNeighbor);
			foreach($dictionary as $key => $value){
				$formatNeighbor = str_replace($value, $key, strtoupper($formatNeighbor));
			}
			
			if (strlen($formatNeighbor) >= 30) {
				$formatNeighbor = substr($formatNeighbor, 0, 30);
			}
		}
		
		return $formatNeighbor;
	}

	public function updateConsumer($params) {
		if (empty($params['CDCLIENTE'])) {
			throw new \Exception("Código do cliente não foi informado.");
		}

		if (empty($params['CDCONSUMIDOR'])) {
			throw new \Exception("Código do consumidor não foi informado.");
		}
		
		$this->consumerFactory->updateConsumer($params);
	}

	const dictionary = array (
		"ACAMP"           => "ACAMPAMENTO",
		"AC"              => "ACESSO",
		"AD"              => "ADRO",
		"ERA"             => "AEROPORTO",
		"AL"              => "ALAMEDA",
		"AT"              => "ALTO",
		"A"               => "AREA",
		"AE"              => "AREA ESPECIAL",
		"AV"              => "AVENIDA",
		"AV-CONT"         => "AVENIDA CONTORNO",
		"BX"              => "BAIXA",
		"BLO"             => "BALAO",
		"BAL"             => "BALNEARIO",
		"BC"              => "BECO",
		"BELV"            => "BELVEDERE",
		"BL"              => "BLOCO",
		"BSQ"             => "BOSQUE",
		"BVD"             => "BOULEVARD",
		"BCO"             => "BURACO",
		"C"               => "CAIS",
		"CALC"            => "CALCADA",
		"CAM"             => "CAMINHO",
		"CPO"             => "CAMPO",
		"CAN"             => "CANAL",
		"CHAP"            => "CHACARA",
		"CHAP"            => "CHAPADAO",
		"CIRC"            => "CIRCULAR",
		"COL"             => "COLONIA",
		"CMP-VR"          => "COMPLEXO VIARIO",
		"COND"            => "CONDOMINIO",
		"CJ"              => "CONJUNTO",
		"COR"             => "CORREDOR",
		"CRG"             => "CORREGO",
		"DSC"             => "DESCIDA",
		"DSV"             => "DESVIO",
		"DT"              => "DISTRITO",
		"EVD"             => "ELEVADA",
		"EQ"              => "ENTRE QUADRA",
		"ESC"             => "ESCADA",
		"ESP"             => "ESPLANADA",
		"ETC"             => "ESTACAO",
		"ESTC"            => "ESTACIONAMENTO",
		"ETD"             => "ESTADIO",
		"ETN"             => "ESTANCIA",
		"EST"             => "ESTRADA",
		"EST-MUN ESTRADA" => "MUNICIPAL",
		"FAV"             => "FAVELA",
		"FAZ"             => "FAZENDA",
		"FRA"             => "FEIRA",
		"FER"             => "FERROVIA",
		"FNT"             => "FONTE",
		"FTE"             => "FORTE",
		"GAL"             => "GALERIA",
		"GJA"             => "GRANJA",
		"HAB"             => "HABITACIONAL",
		"IA"              => "ILHA",
		"JD"              => "JARDIM",
		"JD"			  => "JARDINS",
		"JDE"             => "JARDINETE",
		"LD"              => "LADEIRA",
		"LG"              => "LAGO",
		"LGA"             => "LAGOA",
		"LRG"             => "LARGO",
		"LOT"             => "LOTEAMENTO",
		"MNA"             => "MARINA",
		"MOD"             => "MODULO",
		"TEM"             => "MONTE",
		"MRO"             => "MORRO",
		"NUC"             => "NUCLEO",
		"PDA"             => "PARADA",
		"PAR"             => "PARALELA",
		"PRQ"             => "PARQUE",
		"PSG"             => "PASSAGEM",
		"PSA"             => "PASSARELA",
		"PAS"             => "PASSEIO",
		"PAT"             => "PATIO",
		"PNT"             => "PONTA",
		"PTE"             => "PONTE",
		"PTO"             => "PORTO",
		"PC"              => "PRACA",
		"PR"              => "PRAIA",
		"PRL"             => "PROLONGAMENTO",
		"Q"               => "QUADRA",
		"QTA"             => "QUINTA",
		"QTAS"            => "QUINTA",
		"RMP"             => "RAMPA",
		"REC"             => "RECANTO",
		"RES"             => "RESIDENCIAL",
		"RET"             => "RETA",
		"RER"             => "RETIRO",
		"RTN"             => "RETORNO",
		"ROD-AN"          => "RODO ANEL",
		"ROD"             => "RODOVIA",
		"RTT"             => "ROTATORIA",
		"ROT"             => "ROTULA",
		"R"               => "RUA",
		"ST"              => "SANTO",
		"SIT"             => "SITIO",
		"SUB"             => "SUBIDA",
		"TER"             => "TERMINAL",
		"TV"              => "TRAVESSA",
		"TV-PART"         => "TRAVESSA PARTICULAR",
		"TRV"             => "TRECHO",
		"TRV"             => "TREVO",
		"TCH"             => "TRINCHEIRA",
		"TUN"             => "TUNEL",
		"UNID"            => "UNIDADE",
		"VAL"             => "VALA",
		"VLE"             => "VALE",
		"VER"             => "VEREDA",
		"V"               => "VIA",
		"V-AC"            => "VIA DE ACESSO",
		"V-PED"           => "VIA DE PEDESTRE",
		"V-EVD"           => "VIA ELEVADO",
		"V-EXP"           => "VIA EXPRESSA",
		"VD"              => "VIADUTO",
		"VLA"             => "VIELA",
		"VL"              => "VILA"
	);
}

